<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Payflow
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPayflow extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	
    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		
		$paymentKey = $payment->getKey();
		
		$this->assign('amount', $amount);
		$this->assign('invoice', $invoice);
		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey));
		$this->assign('cancel_url', XiRoute::_("index.php?option=com_payplans&gateway=payflow&view=payment&task=complete&action=cancel&payment_key=".$paymentKey));
		$this->assign('payment_key', $paymentKey);
	  	return $this->_render('form');
	}
	
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
	   $action = JRequest::getVar('action', false);
		if($action == 'cancel'){
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);

		// Set request-specific fields.
		
		$firstName 				= urlencode($data['BILLTOFIRSTNAME']);
		$lastName 				= urlencode($data['BILLTOLASTNAME']);
		$creditCardType 		= urlencode($data['CARDTYPE']);
		$creditCardNumber 		= urlencode(trim($data['ACCT']));
		$cvv2Number 			= urlencode( $data['CVV2']);
		$expDateMonth 	 	    = urlencode(substr($data['EXPDATE'], 0, 2));
		
		// Month must be padded with leading zero
		$padDateMonth 		    = urlencode(str_pad($expDateMonth, 2, '0', STR_PAD_LEFT));
		$expDateYear 		    = urlencode(substr($data['EXPYEAR'], -2));
		$street				    = urlencode($data['BILLTOSTREET']);
		$city 					= urlencode($data['BILLTOCITY']);
		$state 					= urlencode($data['BILLTOSTATE']);
		$zip 					= urlencode($data['BILLTOZIP']);
		$country				= urlencode($data['BILLTOCOUNTRY']);
        $phoneNumber            = urldecode($data['BILLTOPHONENUM']);
        $email                  = urldecode($data['BILLTOEMAIL']);

		// US or other valid country code
		$currencyID 			= urlencode($invoice->getCurrency('isocode'));		// or other currency ('GBP', 'EUR', 'JPY', 'CAD', 'AUD')
		$ipAddress  			= JRequest::getVar('REMOTE_ADDR', NULL ,'SERVER');
		$custom					= $invoice->getKey().'-'.$payment->getKey();
		
	    $root = JURI::root();
    	if(XiFactory::getConfig()->https){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		
		// Add request-specific fields to the request string.
		$nvpStr  			=	"CUSTIP =$ipAddress&CARDTYPE=$creditCardType&ACCT=$creditCardNumber&EXPDATE=$padDateMonth$expDateYear";
		$nvpStr 		   .=	"&CVV2=$cvv2Number&CUSTOM=$custom&VERBOSITY=HIGH";
		
		
		$getApp = XiFactory::getApplication();
		
		$recurring = $invoice->isRecurring(); 
		if($recurring){
			  $nvpStr     .=    "&FIRSTNAME=$firstName&LASTNAME=$lastName&STREET=$street&CITY=$city&STATE=$state&ZIP=$zip";
		      $nvpStr  	  .=	"&COUNTRY=$country&CURRENCYCODE=$currencyID&PHONENUM=$phoneNumber&EMAIL=$email";
		      
		      $this->createRecurringProfile($invoice, $payment, $recurring, $nvpStr,$firstName);
		
		}
		else{

			//non-recurring case
			$paymentType = urlencode('S');	
			$amount      = urlencode($invoice->getTotal());
			$nvpStr     .= "&BILLTOFIRSTNAME=$firstName&BILLTOLASTNAME=$lastName&BILLTOSTREET=$street&BILLTOCITY=$city&BILLTOSTATE=$state&BILLTOZIP=$zip";
		    $nvpStr  	.= "&BILLTOCOUNTRY=$country&CURRENCYCODE=$currencyID&BILLTOPHONENUM=$phoneNumber&BILLTOEMAIL=$email&AMT=$amount&TENDER=C";

			// Execute the API operation; see the executePaypalApi function above.
			$httpParsedResponseAr = $this->executePayflowApi($paymentType, $nvpStr, $payment);
			
			$this->createTransaction($httpParsedResponseAr,$invoice,$payment,'fixed');
	  }
	  // save payment record
		$payment->save();
	  return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	

	// create transaction on the basis of response from payflow
	function createTransaction($httpParsedResponseAr,$invoice,$payment,$type='fixed')
	{
		// when profile creation of fixed payment is there then unique key is stored in PNREF and in case of
		// recurring, it is in RPREF.
		$gatewayTxnId = isset($httpParsedResponseAr["PNREF"]) ? $httpParsedResponseAr["PNREF"] : 0;
		if($type == 'recurring')
		{
			$gatewayTxnId = isset($httpParsedResponseAr["RPREF"]) ? $httpParsedResponseAr["RPREF"] : 0;
		}
	     if(strtoupper($httpParsedResponseAr["RESULT"]) != 0 || $httpParsedResponseAr["RESPMSG"] != "Approved")
			{	
				
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $invoice->getBuyer())
							->set('invoice_id', $invoice->getId())
							->set('payment_id', $payment->getId())
							->set('amount', 0)
							->set('gateway_txn_id', $gatewayTxnId)
							->set('gateway_subscr_id',  0)
							->set('gateway_parent_txn', 0)
							->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAYFLOW_FIXED_PAYMENT_NOT_COMPLETED')
							->set('params', PayplansHelperParam::arrayToIni($httpParsedResponseAr));
							
				$error 		= XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_FIXED_PAYMENT_NOT_COMPLETED');
				$error_code = $httpParsedResponseAr["RESULT"];
				$message 	= urldecode($httpParsedResponseAr["RESPMSG"]);
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $httpParsedResponseAr, 'PayplansPaymentFormatter', '', true);

                // save payment record before redirecting
                $transaction->save();
			    $getApp = XiFactory::getApplication();
				$getApp->enqueueMessage(XiText::_($error.' ('.$error_code.')'),'error');
			  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
			}
			elseif (strtoupper($httpParsedResponseAr["RESULT"]) == 0 && $httpParsedResponseAr["RESPMSG"] == "Approved")
			{
				$transaction = PayplansTransaction::getInstance();
				$transaction->set('user_id', $invoice->getBuyer())
							->set('invoice_id', $invoice->getId())
							->set('payment_id', $payment->getId())
							->set('amount', isset($httpParsedResponseAr['AMT'])?$httpParsedResponseAr['AMT']:0)
							->set('gateway_txn_id', $gatewayTxnId)
							->set('gateway_subscr_id',  0)
							->set('gateway_parent_txn', 0)
							->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAYFLOW_PAYMENT_COMPLETED_SUCCESSFULLY')
							->set('params', PayplansHelperParam::arrayToIni($httpParsedResponseAr));
				
				if(!$transaction->save()){
		         	$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			        PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $httpParsedResponseAr,'PayplansPaymentFormatter', '', true);
		        }
			}

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	function createRecurringProfile($invoice, $payment, $recurring, $nvpStr, $firstName)
	{
		$counter = $invoice->getCounter();
		$paymentType      = urlencode('R');
		$action           = urldecode('A');
		$profilename      = $firstName;
		$regular_amount 		  = urlencode($invoice->getTotal($counter+1)); // Regular Amount
		$now 			  = new XiDate('now');
		$planType 		  = $invoice->isRecurring();
		 if($planType == PAYPLANS_RECURRING_TRIAL_1)
		 {
			// In case of 0 price of trial set optional transaction as A(Authorization) else
			// set it to S(Sales)
		 	$amount      = urlencode($invoice->getTotal());
		 	$optionalTransaction = 'S';
		 	if($amount == '0.00')
		 	{
		 		$optionalTransaction = 'A';
		 	}
		    $nvpStr .= "&OPTIONALTRX=$optionalTransaction&OPTIONALTRXAMT=$amount"; 
		 	$expTime = $invoice->getExpiration(PAYPLANS_RECURRING_TRIAL_1,true);	
		 	$now = $now->addExpiration($expTime,true);
		 }
		 else  {
		 	$now              = $now->addExpiration('000001000000');
		 }
		 $startDate        = urlencode($now->toFormat("%m%d%Y"));
		
		$expirationTime   = $invoice->getExpiration(PAYPLANS_RECURRING);
		$expirationTime   = $this->getRecurrenceTime($expirationTime);
	
		$billingPeriod 	  = urlencode($expirationTime['period']);  
		$billingCycle 	  = urlencode($invoice->getRecurrenceCount());
        
		
		$str            = "TENDER=C&ACTION=$action&PROFILENAME=$profilename&AMT=$regular_amount&START=$startDate&PAYPERIOD=$billingPeriod&TERM=$billingCycle";
		$nvpStr         = $str.$nvpStr;
	
		
		$httpParsedResponseAr = $this->executePayflowApi($paymentType, $nvpStr, $payment);
	
	    $getApp = XiFactory::getApplication();
		
		if(0 == $httpParsedResponseAr["RESULT"] && "Approved" == $httpParsedResponseAr["RESPMSG"] && isset($httpParsedResponseAr['PROFILEID']))
		 {

		   $this->createTransaction($httpParsedResponseAr,$invoice,$payment,'recurring');
		 	$param = new XiParameter();
		 	$param->set('profile_id', urldecode($httpParsedResponseAr['PROFILEID']));
			
			$payment->set('gateway_params',  $param);
				
			if($planType == PAYPLANS_RECURRING){
				$getApp->enqueueMessage(XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_SUBSCRIPTION_ACTIVE_AFTER_ONE_DAY'));		
			}			
					
		}
		else
		{
			$error 		= XiText::_('COM_PAYPLANS_APP_PAYFLOW_RECURRING_PROFILE_REJECTED');
			$error_code = $httpParsedResponseAr["RESULT"];
			$message 	= urldecode($httpParsedResponseAr["RESPMSG"]);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $httpParsedInquiryAr,'PayplansPaymentFormatter', '', true);
			$getApp->enqueueMessage(XiText::_($message.' ('.$error_code.')'),'error');
		  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
	
	}
	
	
	function executePayflowApi($transactionType,$nvpStr_ , $payment)
	{
		// Set up your API credentials, PayPal end point, and API version.
		$API_Vendor   = $this->getAppParam('vendor');
		$API_User     = $this->getAppParam('user',$API_Vendor);
		$API_Partner  = $this->getAppParam('partner');
		$API_Password = $this->getAppParam('password');
		$API_Endpoint = "https://payflowpro.paypal.com";
		
		if($this->getAppParam('sandboxTesting'))
		{	
			$API_Endpoint = "https://pilot-payflowpro.paypal.com";
		}
		$version = urlencode('51.0');
	
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $API_Endpoint);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
	
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 90);
	
		// Set the API operation, version, and API signature in the request.
		//which variable to set $nvpStr_????????????/
		
		$nvpreq = "TRXTYPE=$transactionType&PARTNER=$API_Partner&VENDOR=$API_Vendor&USER=$API_User&PWD=$API_Password&$nvpStr_";
		
		// Set the request as a POST FIELD for curl.
		curl_setopt($ch, CURLOPT_POSTFIELDS, $nvpreq);
	
		// Get response from the server.
	    $httpResponse = curl_exec($ch);
		
		if(!$httpResponse) {
			$error = curl_error($ch).'('.curl_errno($ch).')';
			$message = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_FAILED_MESSAGE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error));
		}

		// Extract the response details.
		$httpResponseAr = explode("&", $httpResponse);
	
		$httpParsedResponseAr = array();
		foreach ($httpResponseAr as $i => $value) {
			$tmpAr = explode("=", $value);
			if(sizeof($tmpAr) > 1) {
				$httpParsedResponseAr[$tmpAr[0]] = $tmpAr[1];
			}
		}
	
		if((0 == sizeof($httpParsedResponseAr)) || !array_key_exists('RESULT', $httpParsedResponseAr)) {
			$error = "Invalid HTTP Response for POST request($nvpreq) to $API_Endpoint.";
			$message = "$methodName_ ".XiText::_('COM_PAYPLANS_APP_PAYPAL_PRO_FAILED_MESSAGE');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error));
		}
	
		return $httpParsedResponseAr;
	}
	
    public function getRecurrenceTime($expTime)
	{
		$expTime['year'] 	= isset($expTime['year']) 	? intval($expTime['year'], 10) : 0;
		$expTime['month'] 	= isset($expTime['month']) ? intval($expTime['month'], 10) : 0;
		$expTime['day'] 	= isset($expTime['day']) 	? intval($expTime['day'], 10)  : 0;
		
		// if years are not empty 
		if(!empty($expTime['year'])){	
		 $year = $expTime['year']; 
		 if($year >= 1)
		 {
			$payPeriod = 'YEAR';
		 }		
		  return array('period' => $payPeriod, 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_YEAR_MESSAGE'));
		}
		
		// if months are not empty 
		if(!empty($expTime['month'])){
			$months = $expTime['month'];
		   if($months == 1)
			{
				$payPeriod = 'MONT';
			}
			if($months >= 3 && $months < 6)
			{
				$payPeriod = 'QTER';
			}
		    if($months >= 6)
			{
				$payPeriod = 'SMYR';
			}
			return array('period' => $payPeriod, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_MONTH_MESSAGE'));
		}
		
		
		if(!empty($expTime['day'])){
			$days  = $expTime['day'];
			$weeks = intval($days/7, 10);
			
		if($weeks == 0)
		{
			$payPeriod = 'DAYS';
		}
		if($weeks == 1)
		{
			$payPeriod = 'WEEK';
		}
		if($weeks == 2 || $weeks == 3)
		{
			$payPeriod = 'BIWK';
		}
		if($weeks == 4)
		{
		    $payPeriod = 'FRWK';
		}	
			return array('period' => $payPeriod, 'unit' => 'Week', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
									'message' => XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_DAY_MESSAGE'));
		}
		
		return false;
	}
	
	
  public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$profileId = $payment->getGatewayParam('profile_id',null);
	
		
		$nvpstr = "ACTION=I&PAYMENTHISTORY=Y&ORIGPROFILEID=$profileId";
		$httpParsedInquiryAr = $this->executePayflowApi(urldecode('R'), $nvpstr ,$payment);
		if($httpParsedInquiryAr['RESULT'] == 0)
		{
			$invoice      = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
			$transcations = $payment->getTransactions();
			
			//Duplicate transaction handling does not required since whether existing transaction exist or not as
			//here we are finding all the transaction of payment and checking in
			// the enquiry response about its existence.
		    foreach ($transcations as $trans)
			 {
			 	$val = $trans->getGatewayTxnId();
			    $pnref_array[] = empty($val)?'':$val;   
			 }
			 $transtate = PAYPLANS_PAYFLOW_SETTLEMENT_SUCCESS;
			 if($this->getAppParam('sandboxTesting'))
			 {
			 	$transtate = PAYPLANS_PAYFLOW_SETTLEMENT_PENDING;
			 }
			 
			foreach ($httpParsedInquiryAr as $key => $value)
			{
				if(strpos($key, 'P_PNREF') !== false && !in_array($httpParsedInquiryAr[$key],$pnref_array))
				{
					$index     = substr($key, -1);
					
					if( 0 === (int)$httpParsedInquiryAr['P_RESULT'.$index] &&  $transtate === (int)$httpParsedInquiryAr['P_TRANSTATE'.$index]
					                                                       && (float)$httpParsedInquiryAr['P_AMT'.$index] > 0 )
					                                                       
					{
						$param = new XiParameter();
						$param->set('P_PNREF',$httpParsedInquiryAr['P_PNREF'.$index]);
						$param->set('P_TRANSTIME', $httpParsedInquiryAr['P_TRANSTIME'.$index]);
						$param->set('P_RESULT', $httpParsedInquiryAr['P_RESULT'.$index]);
						$param->set('P_TRANSLATE',$httpParsedInquiryAr['P_TRANSLATE'].$index);

						$transaction = PayplansTransaction::getInstance();
		                $transaction->set('user_id', $invoice->getBuyer())
									->set('invoice_id', $invoice->getId())
									->set('payment_id', $payment->getId())
									->set('gateway_txn_id', (string)$httpParsedInquiryAr['P_PNREF'.$index])
									->set('gateway_subscr_id', 0)
									->set('amount', $httpParsedInquiryAr['P_AMT'.$index])
									->set('params', PayplansHelperParam::arrayToIni($httpParsedInquiryAr))
									->set('message',XiText::_("COM_PAYPLANS_APP_PAYFLOW_RECURRING_PAYMENT_COMPLETED"));
							
						if(!$transaction->save()){
							$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
							PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $httpParsedInquiryAr,'PayplansPaymentFormatter', '', true);
						}
					   $payment->save();
					}
					
					elseif (in_array((int)$httpParsedInquiryAr['P_TRANSTATE'.$index], array(PAYPLANS_PAYFLOW_ERROR, PAYPLANS_PAYFLOW_FAILED_TO_SETTLE, PAYPLANS_PAYFLOW_SETLEMENT_INCOMPLETE)))
					{
						//terminate the order
						$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
						$invoice->terminate();
						return array();
					}
				}
			}
		}
	}

    public function isSupportPaymentCancellation($invoice)
	{	
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}	
	
	function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$gateway_params = $payment->getGatewayParams()->toArray();
		$profileId = $gateway_params['profile_id'];
	
		$nvpStr = "TENDER=C&ACTION=C&ORIGPROFILEID=$profileId";
		
		$httpParsedResponseAr = $this->executePayflowApi(urldecode('R'), $nvpStr, $payment);
		
		if("0" == $httpParsedResponseAr["RESULT"] || "Approved" == $httpParsedResponseAr["RESPMSG"]) {
			// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('message', 'COM_PAYPLANS_PAYMENT_APP_PAYFLOW_TRANSACTION_RECURRING_CANCEL')	
						->set('gateway_subscr_id', $httpParsedResponseAr['PROFILEID'])
					    ->set('params', PayplansHelperParam::arrayToIni($httpParsedResponseAr))
						->save();
			parent::onPayplansPaymentTerminate($payment, $controller);	
			return $this->_render('cancel_success');
		}
		else {
			$error					= array();			
			$error['error_message'] = XiText::_('COM_PAYPLANS_PAYMENT_APP_PAYFLOW_ERROR_IN_RECURRING_PROFILE_CANCELLATION');
			$error['response']		= $httpParsedResponseAr;
			$message 				= urldecode($httpParsedResponseAr["RESPMSG"]);
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment,$error, 'PayplansPaymentFormatter', '', true);
			
			return $this->_render('cancel_error');
		}
	}
}